#
# This script was originally written by:
# Cristina Lagido, Ian J Wilson, L. Anne Glover, and Jim I Prosser
#
# for the publication: 
# Cristina Lagido, Ian J Wilson, L.Anne Glover, and Jim I Prosser
# A model for bacterial conjugal gene transfer on solid surfaces. 
# FEMS Microbiology Ecology. vol 44. pg 67-78. 2003. 
# http://www3.interscience.wiley.com/journal/118837812/abstract?CRETRY=1&SRETRY=0
#
# It was modified by the TUDelft iGEM 2009 team
# - the original model presented in the paper used an exponentially 
# increasing rate for the colony radius, this was changed to a 
# linear rate of expansion for our model. Several new functions
# were also added.
#
# a=initial.plate() - makes a plate with no transconjugants
#
# b=grow.plate(a,addtime=1,growth=30) - grows plate a for 1 hour
# with a gr of 30 um/hr
#
# plot(b,col=T, leg=T,outline=T) - plots all colonies in color
# with legend and outline of plate shown
# 
# plotdonor(b,col=T, leg=T,outline=T) - plots only donors
# plottrans(b,col=T, leg=T,outline=T) - plots only transconjugants
# plotrecip(b,col=T, leg=T,outline=T) - plots only recipients
# plotreciptrans(b,col=T, leg=T,outline=T) - plots only recipients and transconjugants
#
# c=get.sepdis(b) - gets the receiver to donor/transconjugant separation distances
#
#
#
pr<-function(nd = 10000, nr = 10000,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=12,ngrowth=.99,nmax=1E11)
  # probability of being a recipient cell
  # plateradius and initialsize in mm
{
    cells<-nd+nr; # total cells
    tm<-min(time,(log(nmax)-log(cells))/ngrowth); # effective time, 2.1.6 in paper
    a<-exp(-4*pi*(initialsize+growth*tm)^2*cells/plateradius^2);  # p1(tm), tm=eefctive time, 2.1.4 in paper, modified to be linear
    nr*a/(nd+a*nr); # mentioned in 2.1.5 in paper, the probability that a given colony is a recipient
}
#
equiv <- function(nd,nr,size, plateradius)
  {
    (nd+nr)*size^2/plateradius^2 #Poisson distribution, lambda*|A|, size is r (colony radius)
  }
#
#  Calculation of expected number of transconjugants recipients, and donor colonies
#
etrans <- function(nd,nr,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=8)
{
      nr - erecip(nd,nr,plateradius,initialsize,growth,time);#2.1.6, g_n=0
}

erecip <- function(nd,nr,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=9)
{
    (nd + nr) * prrecip(nd,nr,plateradius,initialsize,growth,time);#2.1.6, g_n=0
}

edonor<- function(nd,nr,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=9)
{
    nd ;  # no change, 2.1.6, g_n=0
}
#
#
#
p0<-function(nd = 10000, nr = 10000,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=9)
{
       exp(-4*pi*(initialsize+growth*tm)^2*(nd+nr)/plateradius^2);# p1(t), 2.1.4 in paper
}

prrecip <- function(nd = 10000, nr = 10000,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=9)
# the probability that a given colony is a recipient
{
    a<-p0(nd,nr,plateradius,initialsize,growth,time)
    nr*a/(nd+nr*a);#the probability that a given colony is a recipient, like pr, but for p1(t) instead of p1(tm)
}
#########################################################################
# 
#   The probability of getting log(N) recipients
#
#########################################################################
logall<-function(lnd= 4,lnr = 4,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=9,ngrowth=.99,nmax=1E11)
{
    ret<-matrix(numeric(6*length(time)*length(lnd)*length(lnr)),ncol=6);
    count<-1;
    for (t in time) {
      for (i in lnd) {
	    for (j in lnr) {
          nd<-10^i;nr<-10^j;
          ret[count,1]<-i;
          ret[count,2]<-j;
          ret[count,3]<-t;
          ret[count,4]<-log10(recipients(nd,nr,plateradius,initialsize,
                                         growth,t,ngrowth,nmax));
          ret[count,5]<-log10(donor(nd,nr,plateradius,initialsize,
                                    growth,t,ngrowth,nmax));
          ret[count,6]<-log10(trans(nd,nr,plateradius,initialsize,
                                    growth,t,ngrowth,nmax));
          count<-count+1;
	    }
      }
    }
    colnames(ret)<-c("lnd","lnr","time","recipients","donor","trans");
    ret;    
}
#########################################################################
recipients<-function(nd = 10000, nr = 10000,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=12,ngrowth=.99,nmax=1E11)
{
    cells<-nd+nr;
    tm<-min(time,(log(nmax)-log(cells))/ngrowth);
    a<-exp(-4*(initialsize+growth*tm)^2*(cells)/plateradius^2);
    pr<-nr*a/(nd+nr*a);
    pr*cells*exp(ngrowth*tm);#2.1.6, M_r
}
#########################################################################
donor<-function(nd = 10000, nr = 10000,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=12,ngrowth=.99,nmax=1E11)
{
    cells<-nd+nr;
    tm<-min(time,(log(nmax)-log(cells))/ngrowth);
    nd*exp(ngrowth*tm);#2.1.6, M_d
}
#########################################################################
trans<-function(nd = 10000, nr = 10000,plateradius = 23.5, 
	initialsize=8E-4,growth=30,time=12,ngrowth=.99,nmax=1E11)
{
    cells<-nd+nr;
    tm<-min(time,(log(nmax)-log(cells))/ngrowth);
    a<-exp(-4*(initialsize+growth*tm)^2*(cells)/plateradius^2);
    pr<-nr*a/(nd+nr*a);
    pt<-1-nd/cells-pr;
    pt*cells*exp(ngrowth*tm);#2.1.6, M_t
}
#########################################################################
plotbw <-function (plate, outline = F, add = F, xlim, ylim,i,colour=F,leg=F, ...) 
{
  if (missing(xlim)) xlim <- c(-plate$plateradius, plate$plateradius)
  if (missing(ylim)) ylim <- c(-plate$plateradius, plate$plateradius)
  if (add != T) {
    plot.new()
    plot.window(xlim, ylim, ...);
  }
  if (outline) {
    #symbols(0,0,circle=plate$plateradius,add=T,inches=F,fg="black")
    curve(sqrt( plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
    curve(-sqrt(plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
  }
  cir <- c(length(plate$recip$x),length(plate$donor$x),length(plate$trans$x));
  ps <- plate$size;
  if (colour) cols <- c(1,3,2)
  else cols <- c(gray(0.9),gray(0),gray(0.6));
  if (leg==T) legend(0.6*plate$plateradius,-0.78*plate$plateradius,c("recipient","donor","transconjugant"),fill=cols,bty="n")
  symbols(plate$recip$x, plate$recip$y, circle=rep(ps,cir[1]),inches=F,add=T, bg=cols[1],fg=cols[1])
  symbols(plate$donor$x, plate$donor$y, circle=rep(ps,cir[2]),inches=F,add=T, bg=cols[2],fg=cols[2])
  symbols(plate$trans$x, plate$trans$y, circle=rep(ps,cir[3]),inches=F,add=T, bg=cols[3],fg=cols[3])
}
#########################################################################
plotrecip <-function (plate, outline = F, add = F, xlim, ylim,i,colour=F,leg=F, ...) 
{
  if (missing(xlim)) xlim <- c(-plate$plateradius, plate$plateradius)
  if (missing(ylim)) ylim <- c(-plate$plateradius, plate$plateradius)
  if (add != T) {
    plot.new()
    plot.window(xlim, ylim, ...);
  }
  if (outline) {
    #symbols(0,0,circle=plate$plateradius,add=T,inches=F,fg="black")
    curve(sqrt( plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
    curve(-sqrt(plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
  }
  cir <- c(length(plate$recip$x),length(plate$donor$x),length(plate$trans$x));
  ps <- plate$size;
  if (colour) cols <- c(1,3,2)
  else cols <- c(gray(0.9),gray(0),gray(0.6));
  if (leg==T) legend(0.6*plate$plateradius,-0.78*plate$plateradius,c("recipient","donor","transconjugant"),fill=cols,bty="n")
  symbols(plate$recip$x, plate$recip$y, circle=rep(ps,cir[1]),inches=F,add=T, bg=cols[1],fg=cols[1])
}
#########################################################################
plotreciptrans <-function (plate, outline = F, add = F, xlim, ylim,i,colour=F,leg=F, ...) 
{
  if (missing(xlim)) xlim <- c(-plate$plateradius, plate$plateradius)
  if (missing(ylim)) ylim <- c(-plate$plateradius, plate$plateradius)
  if (add != T) {
    plot.new()
    plot.window(xlim, ylim, ...);
  }
  if (outline) {
    #symbols(0,0,circle=plate$plateradius,add=T,inches=F,fg="black")
    curve(sqrt( plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
    curve(-sqrt(plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
  }
  cir <- c(length(plate$recip$x),length(plate$donor$x),length(plate$trans$x));
  ps <- plate$size;
  if (colour) cols <- c(1,3,2)
  else cols <- c(gray(0.9),gray(0),gray(0.6));
  if (leg==T) legend(0.6*plate$plateradius,-0.78*plate$plateradius,c("recipient","donor","transconjugant"),fill=cols,bty="n")
  symbols(plate$recip$x, plate$recip$y, circle=rep(ps,cir[1]),inches=F,add=T, bg=cols[1],fg=cols[1])
  symbols(plate$trans$x, plate$trans$y, circle=rep(ps,cir[3]),inches=F,add=T, bg=cols[3],fg=cols[3])
}
#########################################################################
plottrans <-function (plate, outline = F, add = F, xlim, ylim,i,colour=F,leg=F, ...) 
{
  if (missing(xlim)) xlim <- c(-plate$plateradius, plate$plateradius)
  if (missing(ylim)) ylim <- c(-plate$plateradius, plate$plateradius)
  if (add != T) {
    plot.new()
    plot.window(xlim, ylim, ...);
  }
  if (outline) {
    #symbols(0,0,circle=plate$plateradius,add=T,inches=F,fg="black")
    curve(sqrt( plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
    curve(-sqrt(plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
  }
  cir <- c(length(plate$recip$x),length(plate$donor$x),length(plate$trans$x));
  ps <- plate$size;
  if (colour) cols <- c(1,3,2)
  else cols <- c(gray(0.9),gray(0),gray(0.6));
  if (leg==T) legend(0.6*plate$plateradius,-0.78*plate$plateradius,c("recipient","donor","transconjugant"),fill=cols,bty="n")
  symbols(plate$trans$x, plate$trans$y, circle=rep(ps,cir[3]),inches=F,add=T, bg=cols[3],fg=cols[3])
}
#########################################################################
plotdonor <-function (plate, outline = F, add = F, xlim, ylim,i,colour=F,leg=F, ...) 
{
  if (missing(xlim)) xlim <- c(-plate$plateradius, plate$plateradius)
  if (missing(ylim)) ylim <- c(-plate$plateradius, plate$plateradius)
  if (add != T) {
    plot.new()
    plot.window(xlim, ylim, ...);
  }
  if (outline) {
    #symbols(0,0,circle=plate$plateradius,add=T,inches=F,fg="black")
    curve(sqrt( plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
    curve(-sqrt(plate$plateradius^2 - x^2), from = -plate$plateradius, to = plate$plateradius, add = T, lty = 1)
  }
  cir <- c(length(plate$recip$x),length(plate$donor$x),length(plate$trans$x));
  ps <- plate$size;
  if (colour) cols <- c(1,3,2)
  else cols <- c(gray(0.9),gray(0),gray(0.6));
  if (leg==T) legend(0.6*plate$plateradius,-0.78*plate$plateradius,c("recipient","donor","transconjugant"),fill=cols,bty="n")
  symbols(plate$donor$x, plate$donor$y, circle=rep(ps,cir[2]),inches=F,add=T, bg=cols[2],fg=cols[2])
}
#########################################################################
stats.plate<-function(p)
{
    ncr<-length(p$recip$x);
    ncd<-length(p$donor$x);
    nct<-length(p$trans$x);
    tot<-sum(c(ncr,ncd,nct));
    print(paste("recipients",ncr/tot,"donors",ncd/tot,"transconjugants",nct/tot));
    print(paste("recipients",ncr,"donors",ncd,"transconjugants",nct));
}
#########################################################################
#   Simulate an initial set of bacteria on the plate
#########################################################################
initial.plate<-function (nd = 10000, nr = 10000,plateradius = 23.5, 
	initialsize=8E-4,growth=30)
{
    pd<-rcircle(nd,radius=plateradius)
    pr<-rcircle(nr,radius=plateradius)
    plate<-list(recip=pr,donor=pd,trans=list(x=NULL,y=NULL),initialsize=initialsize,
	    size=initialsize,plateradius=plateradius,time=0,growth=growth)
    class(plate)<-"plate"
    plate
}
##########################################################################
grow.plate<-function(plate,addtime=1,growth,ngrowth=.99,nmax=1E11) 
{
    if (missing(growth)) growth<-plate$growth
    plate$size<-plate$size+growth*addtime*0.001
    plate$time<-plate$time+addtime
    get.conjug(plate)
}
##########################################################################
get.conjug<-function(plate)
{
  change<-1
  ##if (length(plate$recip)==0) return( plate);
  while (change==1) {
	change<-0
	oth<-list(
              x=c(plate$donor$x,plate$trans$x),
              y=c(plate$donor$y,plate$trans$y)
              )
	ln <- length(plate$recip$x)
	rem <- -1
	for (i in c(1:ln)) {
      		mn<-min((plate$recip$x[i]-oth$x)^2+(plate$recip$y[i]-oth$y)^2);
		#print(paste("mn",sqrt(mn),"i",i));
      		if (sqrt(mn)<2*plate$size) {
			rem<-c(rem,i)
			change<-1
			#print(paste("change on",i,"sqrt(mn)",sqrt(mn),"platesize",2*plate$size));
      		}
	}
	if (change) {
	    rem<-rem[-1]
	    plate$trans$x<-c(plate$trans$x,plate$recip$x[rem])
	    plate$trans$y<-c(plate$trans$y,plate$recip$y[rem])
	    plate$recip$x<-plate$recip$x[-rem]
	    plate$recip$y<-plate$recip$y[-rem]
      }
    }
    plate;
}
##########################################################################
get.sepdis<-function(plate)
{
  change<-1
  ##if (length(plate$recip)==0) return( plate);
  while (change==1) {
	change<-0
	oth<-list(
              x=c(plate$donor$x,plate$trans$x),
              y=c(plate$donor$y,plate$trans$y)
              )
	ln <- length(plate$recip$x)
	rem <- -1
	mn <- c(1:ln)
	for (i in c(1:ln)) {
      		mn[i]<-min((plate$recip$x[i]-oth$x)^2+(plate$recip$y[i]-oth$y)^2);
		#print(paste("mn",sqrt(mn[i]),"i",i));
      		#if (sqrt(mn[i])<2*plate$size) {
		#	rem<-c(rem,i)
		#	change<-1
		#	#print(paste("change on",i,"sqrt(mn)",sqrt(mn),"platesize",2*plate$size));
      		#}
	}
    }
    mn;
}
##########################################################################
# generate n random points  within a disk
rcircle<-function(n,radius=1)
{
  len<-round(n*4/pi+sqrt(n))
  x<-runif(len,-1,1)
  y<-runif(len,-1,1)
  use<-x^2+y^2<1
  x<-radius*x[use]
  y<-radius*y[use]
  if (length(x)<n)  { # if not enough then repeat
    res<-rcircle(n,radius)
  } else {
    res<-list(x=x[1:n],y=y[1:n])
  }
  res
}
##########################################################################
# get the minimum distance to another point
distances<-function(d)
{
    l<-length(d$y);
    dis<-numeric(l);
    for (i in c(1:l)) 
	dis[i]<-min((d$x[i]-d$x[-i])^2 + (d$y[i]-d$y[-i])^2);
    sqrt(dis);
}
##########################################################################
plot.plate<-function(plate,outline=F,add=F,xlim,ylim,...)
{
  plotbw(plate,outline,add,xlim,ylim,...);
}
##########################################################################
